@@ -211,9 +211,16 @@ etable_selection_change_forall_cb (int row, GalDefineViewsDialog *dialog)
 	if (row != -1) {
 		GalViewCollectionItem *item = gal_view_collection_get_view_item (dialog->collection, row);
 
-		if (item)
+		if (item) {
 			gtk_widget_set_sensitive (glade_xml_get_widget (dialog->gui, "button-delete"),
 						  !item->built_in);
+			if (GAL_VIEW_GET_CLASS (item->view)->edit)
+				gtk_widget_set_sensitive (glade_xml_get_widget (dialog->gui, "button-modify"),
+						item->built_in);
+			else
+				gtk_widget_set_sensitive (glade_xml_get_widget (dialog->gui, "button-modify"),
+						!item->built_in);
+		}
 	}
 }
 
